/* $Workfile: OsaLinuxCallbacks.h $ $Revision: 2 $ $Modtime: 5/17/10 3:27p $  */
/*******************************************************************************

  Odin TeleSystems Inc.

  OTX Telecom Adapter Driver

  @doc internal

  @module

  @normal Copyright (C) Odin TeleSystems Inc., 2007, All Rights Reserved

*******************************************************************************/

#ifndef OSA_LINUX_CALLBACKS_H
#define OSA_LINUX_CALLBACKS_H

#ifdef DAVINCI


#ifdef __cplusplus
extern "C" {
#else
#endif


/*******************************************************************************
   Stuff for running callbacks
*******************************************************************************/

typedef enum {
  OSA_CALLBACK_DEV,     // Used to generate interrupts from Device, e.g. ASP
  OSA_CALLBACK_DSP,     // Used to generate interrupts from DSP
  OSA_CALLBACK_EXTEN,   // Used to generate interrupts from userspace, e.g. TDMoE
} OsaLinuxCallBackType;

/*******************************************************************************
   These functions are called every time
   the device is created and destroyed
   if this device wants to transmit data
   to callback system
*******************************************************************************/
extern int OsaLinuxCallbackEnable( OsaLinuxCallBackType );
extern int OsaLinuxCallbackDisable( OsaLinuxCallBackType );

/*******************************************************************************
   This function is called every time
   device want to transmit or receive data
   Variable method is 't' for TX and is 'r' for RX, 
   It return 0 if process should stop
*******************************************************************************/
extern int OsaLinuxRunCallBack( OsaLinuxCallBackType type, char method, char* buffer, int size ); 


/*******************************************************************************
   Stuff for registering callbacks
*******************************************************************************/

typedef enum {
  OSA_LINUX_CALLBACK_RX = 0x01, // Standard Rx
  OSA_LINUX_CALLBACK_TX = 0x02, // Standard Tx
  OSA_LINUX_CALLBACK_NB = 0x04, // Skip next processing
} OsaLinuxCallBackFlags;

typedef ssize_t (*OsaLinuxCallBackFunc)(int nSourceId, int nBoardType, char * buf, size_t count);

extern int OsaLinuxInitCallback( void );
extern int OsaLinuxDestructCallback( void );

extern int OsaLinuxRegisterCallback( OsaLinuxCallBackFunc func, OsaLinuxCallBackType type, int nFlags );

extern int OsaLinuxDeleteCallback( OsaLinuxCallBackFunc func );

#ifdef __cplusplus
}
#endif

#endif // DAVINCI code only

#endif // OSA_LINUX_CALLBACKS_H

