/* $Workfile: OsaLinuxEthSplitter.c $   $Revision: 3 $    $Modtime: 8/10/10 2:24a $  */
/*******************************************************************************

  Odin TeleSystems Inc.

  OTX Telecom Adapter Driver

  @doc internal

  @module

  @normal Copyright (C) Odin TeleSystems Inc., 2007, All Rights Reserved

*******************************************************************************/

/******************************************************************************
  
  Driver exported entry/exit functions.

  Some of the Linux kernel headers don't like to be compiled with g++ (C++)
  so we need a file that is pure C. This file has two purposes: it implements
  the two Linux module specific entry exit routines, init_module and
  cleanup_module, the file operation callbacks, and also some routines that's
  needed for proc operations.

******************************************************************************/
// This is the only place where this file should be included
// It's unlikely it will be needed anywhere else but if so
// __NO_VERSION__ must be defined before inclusion at the
// other places

// Only include with DaVinci
#ifdef DAVINCI
#if USE_ETH_SPLITTER

#include <linux/version.h>
#include <linux/types.h>
#include <linux/skbuff.h>
#include <linux/netdevice.h>
#include <linux/etherdevice.h>
#include <linux/proc_fs.h>

#include <Osa/Linux/OsaLinuxEthSplitter.h>


//#define DEBUG_ERR 1
//#define DEBUG_PKT 1
#define FRAMES_HAVE_ETHERHEADER 1


typedef struct {
 uint16_t frames_nr;
 uint16_t frame_size[];
} bulkrx_header_t;

uint8_t filter_src_mac[6];
uint8_t filter_dst_mac[6];
uint8_t if_filter_src_mac = 0;
uint8_t if_filter_dst_mac = 0;
uint8_t enabled = 0;
static struct proc_dir_entry* splitter_proc;


#if LINUX_VERSION_CODE < KERNEL_VERSION(2,6,31)
/******************************************************************************
  skb_network_header
******************************************************************************/
static uint8_t *skb_network_header(struct sk_buff *skb)
{
 return skb->nh.raw;
}


#if !FRAMES_HAVE_ETHERHEADER
/******************************************************************************
  skb_set_mac_header
******************************************************************************/
static void skb_set_mac_header(struct sk_buff *skb, const int offset)
{
 skb->mac.raw = skb->data + offset;
}


/******************************************************************************
  skb_mac_header
******************************************************************************/
static uint8_t *skb_mac_header(struct sk_buff *skb)
{
 return skb->mac.raw;
}
#endif
#endif


/******************************************************************************
  Handle
    return 0->OK, Or Error
******************************************************************************/
static int Handle(struct sk_buff *skb, struct net_device *dev)
{
  bulkrx_header_t *h;
  uint32_t total_sz;
  uint32_t n, i;
  uint8_t *d;
  uint32_t mac_len;
  uint32_t mac_pad;
  #if !FRAMES_HAVE_ETHERHEADER
  uint8_t *mac;
  #endif

  if(!pskb_may_pull(skb, 2)) {
    #if DEBUG_ERR
    printk(KERN_ERR "bulkrx_rx: len check 1 failed\n");
    #endif
    return NET_RX_DROP;
  }

  h = (bulkrx_header_t*)skb_network_header(skb);
  n = htons(h->frames_nr);
  if(!pskb_may_pull(skb, 2+2*n)) {
    #if DEBUG_ERR
    printk(KERN_ERR "bulkrx_rx: len check 2 failed\n");
    #endif
    return NET_RX_DROP;
  }

  for(i = 0, total_sz=0; i < n; i++) {
    total_sz += ntohs(h->frame_size[i]);
  }

  if(!pskb_may_pull(skb, 2+2*n+total_sz)) {
    #if DEBUG_ERR
    printk(KERN_ERR "bulkrx_rx: len check 3 failed: skb_len=%u n_frames=%u total_sz=%u\n", skb->len, n, total_sz);
    #endif
    return NET_RX_DROP;
  }

  #if !FRAMES_HAVE_ETHERHEADER
  mac = skb_mac_header(skb);
  mac_len = dev->hard_header_len;
  mac_pad = (4-(mac_len&3))&3;
  #else
  mac_len = 0;
  mac_pad = 2;
  #endif

  #if DEBUG_PKT
  printk(KERN_ERR "bulk_rx: got packet: mac_len = %u n_frames=%u payload=%u\n", mac_len, n, total_sz);
  #endif

  d = (uint8_t*)(h->frame_size+n);
  for(i = 0; i < n; i++) {
    uint16_t l = ntohs(h->frame_size[i]);
    struct sk_buff *newskb;
    newskb = dev_alloc_skb(l+mac_len+mac_pad);
    if(newskb == NULL) 
      return NET_RX_DROP;
    newskb->dev = dev;
    skb_reserve(newskb, mac_pad);
    #if !FRAMES_HAVE_ETHERHEADER
      memcpy(skb_put(newskb, mac_len), mac, mac_len);
      newskb->protocol = eth_type_trans(skb, dev);
      #if DEBUG_PKT
      printk(KERN_ERR "bulkrx_rx: injected packet ip_len=%u\n", l);
      #endif
    #else
      if(l < 14) {
        kfree_skb(newskb);
        break;
      }
      memcpy(skb_put(newskb, l), d, l);
      newskb->protocol = eth_type_trans(skb, dev);
      #if DEBUG_PKT
      printk(KERN_ERR "bulkrx_rx: injected packet ip_len=%u\n", l-14);
      #endif
    #endif
    netif_rx(newskb);
    d += l;
  }
  return NET_RX_SUCCESS;
}


/******************************************************************************
  Check
    return 0->OK, Or Error
******************************************************************************/
static int Check(struct sk_buff *skb, struct net_device *dev)
{
  if (if_filter_src_mac && memcmp(filter_src_mac, skb->mac.raw + ETH_ALEN, ETH_ALEN) != 0) {
    return NET_RX_DROP;
  }
  if (if_filter_dst_mac && memcmp(filter_src_mac, skb->mac.raw, ETH_ALEN) != 0) {
    return NET_RX_DROP;
  }
  return NET_RX_SUCCESS;
}


/******************************************************************************
  Receiver
    return 0->OK, Or Error
******************************************************************************/
static int Receiver(struct sk_buff *skb, struct net_device *dev, struct packet_type *pt
#if LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,31)
                                                        , struct net_device *orig_dev
#endif
                                )
{
  int r = Check(skb, dev);
  if (r == NET_RX_SUCCESS) {
    r = Handle(skb, dev);
  }
  kfree_skb(skb);
  return r;
}


static struct packet_type pa = { 
  .type = 0,
  .dev = NULL,
  .func = Receiver,
};


/******************************************************************************
  OsaLinuxEthSplitterInit
    return 0->OK, Or Error
******************************************************************************/
int OsaLinuxEthSplitterInit(struct OsaLinuxEthSplitterInitS* init)
{
  pa.type = init->eth_type;
  if_filter_src_mac = init->filter_src_mac;
  if (if_filter_src_mac) {
    memcpy(filter_src_mac, init->src_mac, 6);
  }
  if_filter_dst_mac = init->filter_dst_mac;
  if (if_filter_dst_mac) {
    memcpy(filter_dst_mac, init->dst_mac, 6);
  }
  return 0;
}


/******************************************************************************
  OsaLinuxEthSplitterEnable
    return 0->OK, Or Error
******************************************************************************/
int OsaLinuxEthSplitterEnable(void)
{
  if (pa.type) {
    dev_add_pack(&pa);
    enabled = 1;
    return 0;
  }
  return -1;
}


/******************************************************************************
  OsaLinuxEthSplitterDisable
    return 0->OK, Or Error
******************************************************************************/
int OsaLinuxEthSplitterDisable(void)
{
  if (enabled) {
    dev_remove_pack(&pa);
    enabled = 0;
  }
  return 0;
}


/******************************************************************************
  aggr_proc_read
    gives the data to the user
******************************************************************************/
static ssize_t aggr_proc_read(char *buffer, char **start, off_t offset, int size, int *eof, void *data)
{
  int i = 0;
  i += snprintf(buffer+i, size-i, "\n");
  return i;
}


/******************************************************************************
  aggr_proc_write
    takes data from the user
******************************************************************************/
static ssize_t aggr_proc_write(struct file* file, const char* buffer, unsigned long count, void *data)
{
  return count;
}


/******************************************************************************
  OsaLinuxEthSplitterCreateProc
    return 0->OK, Or Error
******************************************************************************/
int OsaLinuxEthSplitterCreateProc(struct proc_dir_entry* proc) 
{
  splitter_proc = create_proc_entry("aggr", 0644, proc);
  if (splitter_proc != NULL) {
    splitter_proc ->owner = THIS_MODULE;
    splitter_proc ->read_proc = aggr_proc_read;
    splitter_proc ->write_proc = aggr_proc_write;
  }
  return 0;
}


/******************************************************************************
  OsaLinuxEthSplitterDestroyProc
    return 0->OK, Or Error
******************************************************************************/
int OsaLinuxEthSplitterDestroyProc(struct proc_dir_entry* proc)
{
  remove_proc_entry("aggr", proc);
  return 0;
}


#endif // USE_ETH_SPLITTER
#endif // DAVINCI
