/* $Workfile: OsaImplementation.cpp $   $Revision: 21 $    $Modtime: 5/26/10 12:11p $  */
/*******************************************************************************

  Odin TeleSystems Inc.

  OTX Telecom Adapter Driver

  @doc internal

  @module

  @normal Copyright (C) Odin TeleSystems Inc., 1998-2010, All Rights Reserved

*******************************************************************************/
#include "Osa.h"
#include "OsaImplementation.h"
#include "OsaString.h"

OsaDriverEntryPointsS  g_sOsaDriverEntryPoints; 
OsaPlatformE gOsaPlatform = OSA_PLATFORM_UNDEFINED;

bool gOsaDriverCompatibilityMode = OSA_FALSE;
char gOsaDriverCompatibilityString[80] = "";
char gOsaThor2PciPlusBuildNumberOverrideString[32] = "";
char gOsaThor8PciPlusBuildNumberOverrideString[32] = "";
OsaUint32 gOsaAsmHighwaySpeedOverride = 0;

OsaUint32 gOsaThor2PciPlusBurstBufferSizeFactor = 0;         
OsaUint32 gOsaThor8PciPlusBurstBufferSizeFactor = 0;         
OsaUint32 gOsaSleipnir1PciPlusBurstBufferSizeFactor = 0;  
OsaUint32 gOsaArniPciPlusBurstBufferSizeFactor = 0;          
OsaUint32 gOsaGimle16PciPlusBurstBufferSizeFactor = 0;       
OsaUint32 gOsaGimle16PciBasic32BurstBufferSizeFactor = 0;    
OsaUint32 gOsaGimle16PciBasic64BurstBufferSizeFactor = 0;    
OsaUint32 gOsaThor2ExpressCardBurstBufferSizeFactor = 0;
OsaUint32 gOsaThor4ExpressCardBurstBufferSizeFactor = 0;
OsaUint32 gOsaAlvisPciExpressBurstBufferSizeFactor = 0;
OsaUint32 gOsaGodarPciExpressBurstBufferSizeFactor = 0;
OsaUint32 gOsaDrvEventLogDisplayLevelFactor = 0;

OsaResultE OsaVerifyBufferSizeFactorValue(OsaUint32 nBurstSizeFactor);  // Verify that the supplied nBurstSizeFactor is a 2^n number

#ifdef __cplusplus
extern "C" {
#endif

extern void OsaSetLogErrorEvent(IN OsaUint32 nLogLevelFactor);

#ifdef __cplusplus
}
#endif

/******************************************************************************
******************************************************************************/
void OsaInit()
{
  // Set all pointers to NULL (assuming NULL is 0x00)
  memset(&g_sOsaDriverEntryPoints, 0x00, sizeof(OsaDriverEntryPointsS)); 
};



/******************************************************************************
******************************************************************************/
OsaUint32 OsaGetAsmHighwaySpeedOverride()
{
  return gOsaAsmHighwaySpeedOverride;
}


/******************************************************************************
******************************************************************************/
OsaResultE OsaSetAsmHighwaySpeedOverride(char *sHwySpeedInKbps)
{

  if (!OsaStrCmp(sHwySpeedInKbps, "2048")) {
    gOsaAsmHighwaySpeedOverride = 2048;
  } else if (!OsaStrCmp(sHwySpeedInKbps, "4096")) {
    gOsaAsmHighwaySpeedOverride = 4096;
  } else if (!OsaStrCmp(sHwySpeedInKbps, "8192")) {
    gOsaAsmHighwaySpeedOverride = 8192;
  } else {
    return OSA_RES_INVALID_PARAMETER;
  }

  return OSA_RES_SUCCESS;

}


/******************************************************************************
******************************************************************************/
OsaUint32 OsaGetThor2PciPlusBurstBufferSizeFactor()        { return gOsaThor2PciPlusBurstBufferSizeFactor; }
OsaUint32 OsaGetThor8PciPlusBurstBufferSizeFactor()        { return gOsaThor8PciPlusBurstBufferSizeFactor; }
OsaUint32 OsaGetSleipnir1PciPlusBurstBufferSizeFactor()    { return gOsaSleipnir1PciPlusBurstBufferSizeFactor; }
OsaUint32 OsaGetArniPciPlusBurstBufferSizeFactor()         { return gOsaArniPciPlusBurstBufferSizeFactor; }
OsaUint32 OsaGetGimle16PciPlusBurstBufferSizeFactor()      { return gOsaGimle16PciPlusBurstBufferSizeFactor; }
OsaUint32 OsaGetGimle16PciBasic32BurstBufferSizeFactor()   { return gOsaGimle16PciBasic32BurstBufferSizeFactor; }
OsaUint32 OsaGetGimle16PciBasic64BurstBufferSizeFactor()   { return gOsaGimle16PciBasic64BurstBufferSizeFactor; }
OsaUint32 OsaGetThor2ExpressCardBurstBufferSizeFactor()    { return gOsaThor2ExpressCardBurstBufferSizeFactor; }
OsaUint32 OsaGetThor4ExpressCardBurstBufferSizeFactor()    { return gOsaThor4ExpressCardBurstBufferSizeFactor; }
OsaUint32 OsaGetAlvisPciExpressBurstBufferSizeFactor()     { return gOsaAlvisPciExpressBurstBufferSizeFactor; }
OsaUint32 OsaGetGodarPciExpressBurstBufferSizeFactor()     { return gOsaGodarPciExpressBurstBufferSizeFactor; }
OsaUint32 OsaGetDrvEventLogDisplayLevelFactor()            { return gOsaDrvEventLogDisplayLevelFactor; }


/******************************************************************************
******************************************************************************/
OsaResultE OsaSetThor2PciPlusBurstBufferSizeFactor(OsaUint32 nBurstSizeFactor)
{
  if (!OSA_SUCCESS(OsaVerifyBufferSizeFactorValue(nBurstSizeFactor))) {
    return OSA_RES_INVALID_PARAMETER; // nBurstSizeFactor must be a 2^n number
  }

  gOsaThor2PciPlusBurstBufferSizeFactor = nBurstSizeFactor;

  return OSA_RES_SUCCESS;

}

OsaResultE OsaSetThor8PciPlusBurstBufferSizeFactor(OsaUint32 nBurstSizeFactor)
{
  if (!OSA_SUCCESS(OsaVerifyBufferSizeFactorValue(nBurstSizeFactor))) {
    return OSA_RES_INVALID_PARAMETER; // nBurstSizeFactor must be a 2^n number
  }

  gOsaThor8PciPlusBurstBufferSizeFactor = nBurstSizeFactor;

  return OSA_RES_SUCCESS;

}

OsaResultE OsaSetSleipnir1PciPlusBurstBufferSizeFactor(OsaUint32 nBurstSizeFactor)
{
  if (!OSA_SUCCESS(OsaVerifyBufferSizeFactorValue(nBurstSizeFactor))) {
    return OSA_RES_INVALID_PARAMETER; // nBurstSizeFactor must be a 2^n number
  }

  gOsaSleipnir1PciPlusBurstBufferSizeFactor = nBurstSizeFactor;

  return OSA_RES_SUCCESS;

}

OsaResultE OsaSetArniPciPlusBurstBufferSizeFactor(OsaUint32 nBurstSizeFactor)
{
  if (!OSA_SUCCESS(OsaVerifyBufferSizeFactorValue(nBurstSizeFactor))) {
    return OSA_RES_INVALID_PARAMETER; // nBurstSizeFactor must be a 2^n number
  }

  gOsaArniPciPlusBurstBufferSizeFactor = nBurstSizeFactor;

  return OSA_RES_SUCCESS;

}

OsaResultE OsaSetGimle16PciPlusBurstBufferSizeFactor(OsaUint32 nBurstSizeFactor)
{
  if (!OSA_SUCCESS(OsaVerifyBufferSizeFactorValue(nBurstSizeFactor))) {
    return OSA_RES_INVALID_PARAMETER; // nBurstSizeFactor must be a 2^n number
  }

  gOsaGimle16PciPlusBurstBufferSizeFactor = nBurstSizeFactor;

  return OSA_RES_SUCCESS;

}

OsaResultE OsaSetGimle16PciBasic32BurstBufferSizeFactor(OsaUint32 nBurstSizeFactor)
{
  if (!OSA_SUCCESS(OsaVerifyBufferSizeFactorValue(nBurstSizeFactor))) {
    return OSA_RES_INVALID_PARAMETER; // nBurstSizeFactor must be a 2^n number
  }

  gOsaGimle16PciBasic32BurstBufferSizeFactor = nBurstSizeFactor;

  return OSA_RES_SUCCESS;

}

OsaResultE OsaSetGimle16PciBasic64BurstBufferSizeFactor(OsaUint32 nBurstSizeFactor)
{
  if (!OSA_SUCCESS(OsaVerifyBufferSizeFactorValue(nBurstSizeFactor))) {
    return OSA_RES_INVALID_PARAMETER; // nBurstSizeFactor must be a 2^n number
  }

  gOsaGimle16PciBasic64BurstBufferSizeFactor = nBurstSizeFactor;

  return OSA_RES_SUCCESS;

}

OsaResultE OsaSetThor2ExpressCardBurstBufferSizeFactor(OsaUint32 nBurstSizeFactor)
{
  if (!OSA_SUCCESS(OsaVerifyBufferSizeFactorValue(nBurstSizeFactor))) {
    return OSA_RES_INVALID_PARAMETER; // nBurstSizeFactor must be a 2^n number
  }

  gOsaThor2ExpressCardBurstBufferSizeFactor = nBurstSizeFactor;

  return OSA_RES_SUCCESS;

}

OsaResultE OsaSetThor4ExpressCardBurstBufferSizeFactor(OsaUint32 nBurstSizeFactor)
{
  if (!OSA_SUCCESS(OsaVerifyBufferSizeFactorValue(nBurstSizeFactor))) {
    return OSA_RES_INVALID_PARAMETER; // nBurstSizeFactor must be a 2^n number
  }

  gOsaThor4ExpressCardBurstBufferSizeFactor = nBurstSizeFactor;

  return OSA_RES_SUCCESS;

}

OsaResultE OsaSetAlvisPciExpressBurstBufferSizeFactor(OsaUint32 nBurstSizeFactor)
{
  if (!OSA_SUCCESS(OsaVerifyBufferSizeFactorValue(nBurstSizeFactor))) {
    return OSA_RES_INVALID_PARAMETER; // nBurstSizeFactor must be a 2^n number
  }

  gOsaAlvisPciExpressBurstBufferSizeFactor = nBurstSizeFactor;

  return OSA_RES_SUCCESS;

}

OsaResultE OsaSetGodarPciExpressBurstBufferSizeFactor(OsaUint32 nBurstSizeFactor)
{
  if (!OSA_SUCCESS(OsaVerifyBufferSizeFactorValue(nBurstSizeFactor))) {
    return OSA_RES_INVALID_PARAMETER; // nBurstSizeFactor must be a 2^n number
  }

  gOsaGodarPciExpressBurstBufferSizeFactor = nBurstSizeFactor;

  return OSA_RES_SUCCESS;

}

OsaResultE OsaSetDrvEventLogDisplayLevelFactor(OsaUint32 nLogLevelFactor)
{

  gOsaDrvEventLogDisplayLevelFactor = nLogLevelFactor;

  OsaSetLogErrorEvent(nLogLevelFactor);

  return OSA_RES_SUCCESS;

}

/******************************************************************************
******************************************************************************/
OsaResultE OsaVerifyBufferSizeFactorValue(OsaUint32 nBurstSizeFactor)
{
  OsaUint32 temp = nBurstSizeFactor;

  // Verify that the supplied nBurstSizeFactor is a 2^n number
  while (temp > 1) {
    if ((temp % 2) != 0) {
      OSA_EPRINTF(("Invalid nBurstSizeFactor (0x%X). Must be a 2^n number\n", nBurstSizeFactor));
      return OSA_RES_INVALID_PARAMETER; // nBurstSizeFactor must be a 2^n number
    }
    temp = temp/2;
  }

  return OSA_RES_SUCCESS;
}


/******************************************************************************
******************************************************************************/
bool OsaGetDriverCompatibilityMode()
{
  return gOsaDriverCompatibilityMode;
}


/******************************************************************************
******************************************************************************/
OsaResultE OsaGetCompatibilityVersion(char *sVersion, OsaUint32 bufsiz)
{
  OsaUint32 ui32;

  ui32 = OsaStrLen(gOsaDriverCompatibilityString);
  if ((ui32 + 1) > bufsiz) {
    return OSA_RES_INVALID_BUFFER_SIZE;
  }

  if (!sVersion) {
    return OSA_RES_FAILED;
  }

  OsaStrCpy(sVersion, gOsaDriverCompatibilityString);

  return OSA_RES_SUCCESS;
}


/******************************************************************************
******************************************************************************/
OsaResultE OsaGetThor2PciPlusBuildNumberOverride(char *sVersion, OsaUint32 bufsiz)
{
  OsaUint32 ui32;

  ui32 = OsaStrLen(gOsaThor2PciPlusBuildNumberOverrideString);
  if ((ui32 + 1) > bufsiz) {
    return OSA_RES_INVALID_BUFFER_SIZE;
  }

  if (!sVersion) {
    return OSA_RES_FAILED;
  }

  OsaStrCpy(sVersion, gOsaThor2PciPlusBuildNumberOverrideString);

  if (sVersion[0] == '\0') {
    // Empty Build string. Not a valid build string.
    return OSA_RES_FAILED;
  }

  return OSA_RES_SUCCESS;
}


/******************************************************************************
******************************************************************************/
OsaResultE OsaGetThor8PciPlusBuildNumberOverride(char *sVersion, OsaUint32 bufsiz)
{
  OsaUint32 ui32;

  ui32 = OsaStrLen(gOsaThor8PciPlusBuildNumberOverrideString);
  if ((ui32 + 1) > bufsiz) {
    return OSA_RES_INVALID_BUFFER_SIZE;
  }

  if (!sVersion) {
    return OSA_RES_FAILED;
  }

  OsaStrCpy(sVersion, gOsaThor8PciPlusBuildNumberOverrideString);

  if (sVersion[0] == '\0') {
    // Empty Build string. Not a valid build string.
    return OSA_RES_FAILED;
  }

  return OSA_RES_SUCCESS;
}


/******************************************************************************
******************************************************************************/
OsaResultE OsaSetCompatibilityVersion(char *sVersion)
{
  OsaUint32 ui32;

  if (sVersion) {
    ui32 = OsaStrLen(sVersion);
    if ((ui32 + 1) > sizeof(gOsaDriverCompatibilityString)) {
      return OSA_RES_INVALID_BUFFER_SIZE;
    }

    gOsaDriverCompatibilityMode = OSA_TRUE;
    OsaStrCpy(gOsaDriverCompatibilityString, sVersion);
    OSA_DPRINTF(("OTX Driver CompatiblityVersion = %s\n", gOsaDriverCompatibilityString));
  } else {
    gOsaDriverCompatibilityMode = OSA_FALSE;
    OsaStrCpy(gOsaDriverCompatibilityString, "");
  }

  return OSA_RES_SUCCESS;
}


/******************************************************************************
******************************************************************************/
OsaResultE OsaSetThor2PciPlusBuildNumberOverride(char *sBuildNumber)
{
  OsaUint32 ui32;

  if (sBuildNumber) {
    ui32 = OsaStrLen(sBuildNumber);
    if ((ui32 + 1) > sizeof(gOsaDriverCompatibilityString)) {
      return OSA_RES_INVALID_BUFFER_SIZE;
    }

    OsaStrCpy(gOsaThor2PciPlusBuildNumberOverrideString, sBuildNumber);
    OSA_DPRINTF(("gOsaThor2PciPlusBuildNumberOverrideString = %s\n", gOsaThor2PciPlusBuildNumberOverrideString));
  } else {
    OsaStrCpy(gOsaThor2PciPlusBuildNumberOverrideString, "");
  }

  return OSA_RES_SUCCESS;
}


/******************************************************************************
******************************************************************************/
OsaResultE OsaSetThor8PciPlusBuildNumberOverride(char *sBuildNumber)
{
  OsaUint32 ui32;

  if (sBuildNumber) {
    ui32 = OsaStrLen(sBuildNumber);
    if ((ui32 + 1) > sizeof(gOsaDriverCompatibilityString)) {
      return OSA_RES_INVALID_BUFFER_SIZE;
    }

    OsaStrCpy(gOsaThor8PciPlusBuildNumberOverrideString, sBuildNumber);
    OSA_DPRINTF(("gOsaThor8PciPlusBuildNumberOverrideString = %s\n", gOsaThor8PciPlusBuildNumberOverrideString));
  } else {
    OsaStrCpy(gOsaThor8PciPlusBuildNumberOverrideString, "");
  }

  return OSA_RES_SUCCESS;
}


/******************************************************************************
******************************************************************************/
bool OsaCheckCallbacks()
{
  if (
    !g_sOsaDriverEntryPoints.pfnUnload          ||     
    !g_sOsaDriverEntryPoints.pfnAddBoard        ||      
    !g_sOsaDriverEntryPoints.pfnStartBoard      ||    
    !g_sOsaDriverEntryPoints.pfnBoardState      ||    
    !g_sOsaDriverEntryPoints.pfnRemoveBoard     ||   
    !g_sOsaDriverEntryPoints.pfnOpenDevice      ||    
    !g_sOsaDriverEntryPoints.pfnCloseDevice     ||   
    !g_sOsaDriverEntryPoints.pfnIoControlDevice
    )
  {
    return false;
  } else {
    return true;
  }
}

/******************************************************************************
******************************************************************************/
void OsaSetUnloadCallback(OsaUnloadFnP pfnCallback)
{
  g_sOsaDriverEntryPoints.pfnUnload = pfnCallback;
}


/******************************************************************************
******************************************************************************/
void OsaSetAddBoardCallback(OsaAddBoardFnP pfnCallback)
{
  g_sOsaDriverEntryPoints.pfnAddBoard = pfnCallback;
}


/******************************************************************************
******************************************************************************/
void OsaSetStartBoardCallback(OsaStartBoardFnP pfnCallback)             
{
  g_sOsaDriverEntryPoints.pfnStartBoard = pfnCallback;
}


/******************************************************************************
******************************************************************************/
void OsaSetBoardStateCallback(OsaBoardStateFnP pfnCallback)
{
  g_sOsaDriverEntryPoints.pfnBoardState = pfnCallback;
}


/******************************************************************************
******************************************************************************/
void OsaSetRemoveBoardCallback(OsaRemoveBoardFnP pfnCallback)
{
  g_sOsaDriverEntryPoints.pfnRemoveBoard = pfnCallback;
}


/******************************************************************************
******************************************************************************/
void OsaSetOpenDeviceCallback(OsaOpenDeviceFnP pfnCallback)
{
  g_sOsaDriverEntryPoints.pfnOpenDevice = pfnCallback;
}


/******************************************************************************
******************************************************************************/
void OsaSetCloseDeviceCallback(OsaCloseDeviceFnP pfnCallback)
{
  g_sOsaDriverEntryPoints.pfnCloseDevice = pfnCallback;
}


/******************************************************************************
******************************************************************************/
void OsaSetIoControlDeviceCallback(OsaIoControlDeviceFnP pfnCallback)
{
  g_sOsaDriverEntryPoints.pfnIoControlDevice = pfnCallback;
}


/******************************************************************************
  See also struct OsaTransStruct and macro OSA_MK_TRANS_STRUCT()
******************************************************************************/
const char *OsaFindTextForCode(OsaTransStruct sqTextCodeMappings[], OsaUint32 nCode) 
{
  OsaTransStruct *pTransStruct = sqTextCodeMappings;
  while (pTransStruct->pszText) {
    if (pTransStruct->nCode == nCode) {
      return pTransStruct->pszText;
    }
    pTransStruct++;
  }
  return "??";
}


/******************************************************************************
******************************************************************************/
const OsaString &OsaGetExternalDeviceName(OsaExternalDeviceRep externalDeviceRep)
{
  OsaDevice *pExternalDevice = reinterpret_cast<OsaDevice *>(externalDeviceRep);

  OSA_ASSERT(pExternalDevice);

  return pExternalDevice->m_strName;
}


/******************************************************************************
******************************************************************************/
OsaUint32 OsaGetHwLocationAddress(IN OsaExternalDeviceRep externalDeviceRep)
{
  OsaDevice *pExternalDevice = reinterpret_cast<OsaDevice *>(externalDeviceRep);

  OSA_ASSERT(pExternalDevice);

  return pExternalDevice->m_nHwLocationAddress;
}

/******************************************************************************
******************************************************************************/
OsaInt32 OsaGetPcmciaSocketNumber(IN OsaExternalDeviceRep externalDeviceRep)
{
  OsaDevice *pExternalDevice = reinterpret_cast<OsaDevice *>(externalDeviceRep);

  OSA_ASSERT(pExternalDevice);

  return pExternalDevice->m_nPcmciaSocketNumber;
}

/******************************************************************************
******************************************************************************/
OsaInt32 OsaGetPcmciaControllerNumber(IN OsaExternalDeviceRep externalDeviceRep)
{
  OsaDevice *pExternalDevice = reinterpret_cast<OsaDevice *>(externalDeviceRep);

  OSA_ASSERT(pExternalDevice);

  return pExternalDevice->m_nPcmciaControllerNumber;
}

/******************************************************************************
******************************************************************************/
#if DBG
static char DbgDumpBufferAsciiTbl[256] = {
//          0    1    2    3    4    5    6    7      8    9    a    b    c    d    e    f
/* 00 */   '.', '.', '.', '.', '.', '.', '.', '.',   '.', '.', '.', '.', '.', '.', '.', '.', 
/* 10 */   '.', '.', '.', '.', '.', '.', '.', '.',   '.', '.', '.', '.', '.', '.', '.', '.', 
/* 20 */   ' ', '!','\"', '#', '$', '%', '&', '\'',  '(', ')', '*', '+', ',', '-', '.', '/', 
/* 30 */   '0', '1', '2', '3', '4', '5', '6', '7',   '8', '9', ':', ';', '<', '=', '>', '?', 
/* 40 */   '@', 'A', 'B', 'C', 'D', 'E', 'F', 'G',   'H', 'I', 'J', 'K', 'L', 'M', 'N', 'O',
/* 50 */   'P', 'Q', 'R', 'S', 'T', 'U', 'V', 'W',   'X', 'Y', 'Z', '[','\\', ']', '^', '_',
/* 60 */   '`', 'a', 'b', 'c', 'd', 'e', 'f', 'g',   'h', 'i', 'j', 'k', 'l', 'm', 'n', 'o',
/* 70 */   'p', 'q', 'r', 's', 't', 'u', 'v', 'w',   'x', 'y', 'z', '{', '|', '}', '~', '.',
/* 80 */   '.', '.', '.', '.', '.', '.', '.', '.',   '.', '.', '.', '.', '.', '.', '.', '.', 
/* 90 */   '.', '.', '.', '.', '.', '.', '.', '.',   '.', '.', '.', '.', '.', '.', '.', '.', 
/* a0 */   '.', '.', '.', '.', '.', '.', '.', '.',   '.', '.', '.', '.', '.', '.', '.', '.', 
/* b0 */   '.', '.', '.', '.', '.', '.', '.', '.',   '.', '.', '.', '.', '.', '.', '.', '.', 
/* c0 */   '.', '.', '.', '.', '.', '.', '.', '.',   '.', '.', '.', '.', '.', '.', '.', '.', 
/* d0 */   '.', '.', '.', '.', '.', '.', '.', '.',   '.', '.', '.', '.', '.', '.', '.', '.', 
/* e0 */   '.', '.', '.', '.', '.', '.', '.', '.',   '.', '.', '.', '.', '.', '.', '.', '.', 
/* f0 */   '.', '.', '.', '.', '.', '.', '.', '.',   '.', '.', '.', '.', '.', '.', '.', '.'
};

#if defined(_TURNSTONE_)
__declspec(dllimport) int __cdecl sprintf(char *, const char *, ...);
#else
extern "C" OSA_STR_RESULT OSA_STR_PRINTF(char *, size_t cchDest, const char *, ...);
#endif

void OsaDbgDumpBuffer(char *buf, int siz)
{
  int i, j;
  char s[82], *cp;

  for (i = 0; i < siz; i += 16) {

    // format a single line
    cp = s;

    OSA_STR_SPRINTF(cp, 82, "%05x   ", i - (i % 16));
    cp += OsaStrLen(cp);

    // dump hex representation
    for (j = 0; j < 16; j++) {
      if (j == 8) {
        // separator
        OsaStrCpy(cp, "  ");
        cp += OsaStrLen(cp);
      }
      if (i + j < siz) {
        OSA_STR_SPRINTF(cp, 82, "%02x ", (unsigned int) (unsigned char) buf[i + j]);
        cp += OsaStrLen(cp);
      } else {
        OsaStrCpy(cp, "   ");
        cp += OsaStrLen(cp);
      }
    }

    // separator
    OsaStrCpy(cp, "   ");
    cp += OsaStrLen(cp);

    // ascii representation
    for (j = 0; j < 16; j++) {
      if (i + j < siz) {
        *cp++ = DbgDumpBufferAsciiTbl[(unsigned char) buf[i + j]];
      } else {
        *cp++ = ' ';
      }
    }

    *cp = '\0';

    OSA_DPRINTF(("%s\n", s));
  }
}

void OsaDbgDumpWordBuffer(OTX_UINT16 *buf, OTX_UINT32 addr, int siz)
{
  int i, j;
  char s[164], *cp;

  for (i = 0; i < siz; i += 16) {

    // format a single line
    cp = s;

    OSA_STR_SPRINTF(cp, 164, "%08x   ", addr + i);
    cp += OsaStrLen(cp);

    // dump hex representation
    for (j = 0; j < 16; j++) {
      if (j == 8) {
        // separator
        OsaStrCpy(cp, "  ");
        cp += OsaStrLen(cp);
      }
      if (i + j < siz) {
        OSA_STR_SPRINTF(cp, 164, "%04x ", buf[i + j]);
        cp += OsaStrLen(cp);
      } else {
        OsaStrCpy(cp, "     ");
        cp += OsaStrLen(cp);
      }
    }

    *cp = '\0';

    OSA_DPRINTF(("%s\n", s));
  }
}

#endif  // #if DBG

